import{j as e}from"./@tanstack-D1KSn3AD.js";import{B as t,ao as n}from"./index-M2LMbZSu.js";import{I as s}from"./Infobox-BdBpSCoi.js";import{H as o}from"./Helpbox-CmVmH8G7.js";import{g as i}from"./Commands-jfwCnkIp.js";import"./@radix-Mqgm9uyP.js";import"./@react-router-t1nY6c48.js";import"./@reactflow-DOgoC88E.js";import"./help-CRVM-AXC.js";import"./CodeSnippet-DnKTf_S2.js";import"./copy-Vxjjuv3p.js";const c=[{command:`zenml secret create SECRET_NAME --key1=value1 --key2=value2 --key3=value3 ...
            

# Another option is to use the '--values' option and provide key-value pairs in either JSON or YAML format.
zenml secret create SECRET_NAME --values '{"key1":"value2","key2":"value2","key3":"value3"}'`,description:"Create a secret"},{command:"zenml secret list",description:"List all the available secrets"},{command:"zenml secret get SECRET_NAME",description:"Get the key-value pairs for a particular secret"},{command:"zenml secret delete SECRET_NAME",description:"Delete a secret that you have previously registered"}];function l(){return e.jsx(s,{children:e.jsxs("div",{className:"flex w-full flex-wrap items-center gap-x-2 gap-y-0.5 text-text-md",children:[e.jsx("p",{className:"font-semibold",children:"We are creating a new Secrets experience"}),e.jsx("p",{children:"In the meanwhile you can use the CLI to add and manage your secrets."})]})})}function m(){return e.jsx(t,{className:"flex flex-col-reverse items-stretch overflow-hidden md:flex-row",children:e.jsxs("div",{className:"w-full p-7 md:w-2/3",children:[e.jsx("h2",{className:"text-display-xs font-semibold",children:"Learn More about Secrets"}),e.jsx("p",{className:"mt-2 text-text-lg text-theme-text-secondary",children:"Learn about using ZenML Secrets for a secure, centralized management of your pipeline secrets and configurations."})]})})}function d(){return e.jsxs("section",{className:"space-y-5 pl-8 pr-5",children:[e.jsx(s,{className:"text-text-md",intent:"neutral",children:"A ZenML Secret is a collection or grouping of key-value pairs stored by the ZenML secrets store. ZenML Secrets are identified by a unique name which allows you to fetch or reference them in your pipelines and stacks."}),c.map((r,a)=>e.jsx(e.Fragment,{children:i(r)},a)),e.jsx(o,{text:"Check all the commands and find more about Secrets in our Docs",link:"https://docs.zenml.io/user-guide/advanced-guide/secret-management"})]})}function E(){return e.jsxs(t,{className:"space-y-4 p-5",children:[e.jsx("h1",{className:"text-text-xl font-semibold",children:"Secrets"}),e.jsx(l,{}),e.jsx(m,{}),e.jsxs("div",{className:"flex items-center gap-2",children:[e.jsx(n,{}),"Administering your Secrets"]}),e.jsx(d,{})]})}export{E as default};
