import{j as e}from"./@tanstack-D1KSn3AD.js";import{P as r}from"./PageHeader-hJZEYC_E.js";import{B as a,o as n}from"./index-M2LMbZSu.js";import{C as l}from"./CodeSnippet-DnKTf_S2.js";import{I as i}from"./Infobox-BdBpSCoi.js";import{f as m}from"./@react-router-t1nY6c48.js";import{C as d,m as c}from"./cloud-only-BPeZhhgU.js";import"./@radix-Mqgm9uyP.js";import"./@reactflow-DOgoC88E.js";import"./copy-Vxjjuv3p.js";function x(){return e.jsx(i,{children:e.jsxs("div",{className:"flex w-full flex-wrap items-center gap-x-2 gap-y-0.5 text-text-md",children:[e.jsx("p",{className:"font-semibold",children:"This is a ZenML Cloud feature"}),e.jsx("p",{children:"Upgrade to ZenML Cloud to access the Model Control Plane and interact with your models in the Dashboard."})]})})}function p(){const[o]=m(),s=o.get("model");function t(){return s?`zenml model list --name='contains:${s}'`:"zenml model list"}return e.jsxs(a,{className:"flex flex-wrap justify-between p-5",children:[e.jsxs("div",{children:[e.jsx("p",{className:"text-text-xl font-semibold",children:"Staying Open Source? "}),e.jsx("p",{className:"text-theme-text-secondary",children:"No problem! Use this CLI command to list your models"})]}),e.jsx(l,{code:t()})]})}const f="/assets/mcp-ocoXuQwQ.png";function w(){return e.jsxs("div",{children:[e.jsxs(r,{className:"flex items-center gap-1",children:[e.jsx("h1",{className:"text-display-xs font-semibold",children:"Models"}),e.jsx(n,{color:"purple",rounded:!0,size:"sm",children:e.jsx("span",{className:"font-semibold text-primary-500",children:"Cloud"})})]}),e.jsxs("div",{className:"layout-container space-y-5 py-5",children:[e.jsx(x,{}),e.jsx(d,{image:{src:f,alt:"Screenshot of the ZenML Cloud Model Control plane"},features:c}),e.jsx(p,{})]})]})}export{w as default};
