import{j as e}from"./@tanstack-D1KSn3AD.js";import{B as o,ao as i}from"./index-M2LMbZSu.js";import{I as s}from"./Infobox-BdBpSCoi.js";import{H as n}from"./Helpbox-CmVmH8G7.js";import{g as a}from"./Commands-jfwCnkIp.js";import"./@radix-Mqgm9uyP.js";import"./@react-router-t1nY6c48.js";import"./@reactflow-DOgoC88E.js";import"./help-CRVM-AXC.js";import"./CodeSnippet-DnKTf_S2.js";import"./copy-Vxjjuv3p.js";const c=[{command:"zenml code-repository register <NAME> --type=<CODE_REPOSITORY_TYPE] [--CODE_REPOSITORY_OPTIONS]",description:"Register a code repository"},{command:"zenml code-repository register <NAME> --type=custom --source=<CODE_REPOSITORY_SOURCE> [--CODE_REPOSITORY_OPTIONS]",description:"Use a custom repository "},{command:"zenml code-repository list",description:"List your registered code repositories"},{command:"zenml code-repository delete <REPOSITORY_NAME_OR_ID>",description:"Delete a code repository that you have previously registered"}];function d(){return e.jsx(s,{children:e.jsxs("div",{className:"flex w-full flex-wrap items-center gap-x-2 gap-y-0.5 text-text-md",children:[e.jsx("p",{className:"font-semibold",children:"We are creating a new Repositories experience"}),e.jsx("p",{children:"In the meanwhile you can use the CLI to add and manage your repos."})]})})}function m(){return e.jsx(o,{className:"flex flex-col-reverse items-stretch overflow-hidden md:flex-row",children:e.jsxs("div",{className:"w-full p-7 md:w-2/3",children:[e.jsx("h2",{className:"text-display-xs font-semibold",children:"Learn More about Repositories"}),e.jsx("p",{className:"mt-2 text-text-lg text-theme-text-secondary",children:"Get started with ZenML Repositories for streamlined pipeline versioning and faster Docker builds."})]})})}function l(){return e.jsxs("section",{className:"space-y-5 pl-8 pr-5",children:[e.jsx(s,{className:"text-text-md",intent:"neutral",children:"Code repositories enable ZenML to keep track of the code version that you use for your pipeline runs. Additionally, running a pipeline which is tracked in a registered code repository can decrease the time it takes Docker to build images for containerized stack components."}),c.map((t,r)=>e.jsx(e.Fragment,{children:a(t)},r)),e.jsx(n,{text:"Check all the commands and find more about Repositories in our Docs",link:"https://docs.zenml.io/user-guide/production-guide/connect-code-repository"})]})}function E(){return e.jsxs(o,{className:"space-y-4 p-5",children:[e.jsx("h1",{className:"text-text-xl font-semibold",children:"Repositories"}),e.jsx(d,{}),e.jsx(m,{}),e.jsxs("div",{className:"flex items-center gap-2",children:[e.jsx(i,{}),"Administering your Code Repositories"]}),e.jsx(l,{})]})}export{E as default};
