import{j as n,F as o,k as i,l as u}from"./index-M2LMbZSu.js";import{b as c}from"./@tanstack-D1KSn3AD.js";async function p(e){const r=i(u.currentUser),t=await n(r,{method:"PUT",headers:{"Content-Type":"application/json"},body:JSON.stringify(e)});if(!t.ok){const s=await t.json().then(a=>a.detail instanceof Array?a.detail[1]:a.detail).catch(()=>"Failed to update User");throw new o({status:t.status,statusText:t.statusText,message:s})}return t.json()}function d(e){return c({mutationFn:async r=>p(r),...e})}export{d as u};
