import{j as t}from"./@tanstack-D1KSn3AD.js";import{r as n}from"./@radix-Mqgm9uyP.js";import{z as r,B as c,h as m,r as u}from"./index-M2LMbZSu.js";import{a as p}from"./UpdatePasswordSchemas-DGDVsTXq.js";import{S as d}from"./check-circle-B_YRDK8p.js";import{f as x,L as h}from"./@react-router-t1nY6c48.js";const i=n.createContext(null);function N({children:e,initialStep:s=1}){const[o,a]=n.useState(s);return t.jsx(i.Provider,{value:{surveyStep:o,setSurveyStep:a},children:e})}function f(){const e=n.useContext(i);if(e===null)throw new Error("useSurveyContext must be used within an SurveyProvider");return e}function P({stepAmount:e}){const{surveyStep:s}=f();return s>e?null:t.jsx("ol",{className:"flex flex-wrap justify-center gap-1 pb-5","aria-label":"progress",children:Array.from({length:e},(o,a)=>t.jsx("li",{"aria-current":s===a+1?"step":void 0,className:`h-0.5 w-[90px] rounded-rounded ${s===a+1?"bg-primary-100":s>a?"bg-primary-300":"bg-neutral-200"}`},a))})}const L=r.object({fullName:r.union([r.string(),r.literal("")]),email:r.union([r.string().email(),r.literal("")]),getUpdates:r.boolean()}).refine(e=>e.getUpdates?e.email!=="":!0),B=r.object({primaryUse:r.string().min(1),amountProductionModels:r.string().min(1)}),E=r.object({channels:r.string().array().min(1),other:r.boolean(),otherVal:r.string().optional()}).refine(e=>e.other?e.otherVal!=="":!0),F=r.object({serverName:r.string().optional()});function g(e=!1){return p.extend({username:r.string()}).refine(s=>s.newPassword===s.confirmPassword,{path:["confirmPassword"],message:"Passwords do not match"}).refine(s=>e?s.username.length>0:!0)}g();const S=e=>n.createElement("svg",{viewBox:"0 0 24 24",fill:"black",xmlns:"http://www.w3.org/2000/svg",...e},n.createElement("path",{fillRule:"evenodd",clipRule:"evenodd",d:"M11.2929 4.29289C11.6834 3.90237 12.3166 3.90237 12.7071 4.29289L19.7071 11.2929C20.0976 11.6834 20.0976 12.3166 19.7071 12.7071L12.7071 19.7071C12.3166 20.0976 11.6834 20.0976 11.2929 19.7071C10.9024 19.3166 10.9024 18.6834 11.2929 18.2929L16.5858 13H5C4.44772 13 4 12.5523 4 12C4 11.4477 4.44772 11 5 11H16.5858L11.2929 5.70711C10.9024 5.31658 10.9024 4.68342 11.2929 4.29289Z"}));function R({username:e,subHeader:s,displayBody:o=!0}){const[a]=x(),l=a.get("redirect");return t.jsxs(c,{className:"flex max-w-[540px] flex-col items-center justify-center space-y-7 px-7 py-9",children:[t.jsx(d,{className:"h-[120px] w-[120px] fill-theme-text-success"}),t.jsxs("div",{className:"space-y-3 text-center",children:[t.jsxs("p",{className:"text-display-xs font-semibold",children:["Congratulations!",t.jsx("br",{}),s]}),o&&t.jsxs("p",{className:"text-theme-text-secondary",children:["You can log in to the dashboard with your username"," ",t.jsx("span",{className:"font-semibold text-theme-text-primary",children:e})," and your password to start exploring!"]}),t.jsx(m,{className:"inline-flex",size:"md",intent:"primary",asChild:!0,children:t.jsxs(h,{to:l||u.home,children:[t.jsx("span",{children:"Go to Dashboard"}),t.jsx(S,{className:"h-5 w-5 fill-white"})]})})]})]})}export{E as A,P as S,R as a,N as b,F as c,L as d,g,B as p,f as u};
